use crate::{
    endpoints::area::NewOrEditAreaTemplate,
    utils::{ApplicationError, TemplateResponse},
};
use actix_web::{web, Responder};
use brass_db::models::{Role, User};

#[actix_web::get("/area/new")]
async fn get(user: web::ReqData<User>) -> Result<impl Responder, ApplicationError> {
    if user.role != Role::Admin {
        return Err(ApplicationError::Unauthorized);
    }

    let template = NewOrEditAreaTemplate {
        user: user.into_inner(),
        area: None,
    };

    Ok(template.to_response()?)
}

#[cfg(test)]
mod tests {
    use crate::{
        utils::test_helper::{
            assert_snapshot, read_body, test_get, DbTestContext, RequestConfig, StatusCode,
        },
    };
    use brass_db::models::{Function, Role};
    use brass_macros::db_test;

    #[db_test]
    async fn produces_template_when_user_is_admin(context: &DbTestContext) {
        let app = context.app().await;
        let config = RequestConfig {
            uri: "/area/new".to_string(),
            role: Role::Admin,
            function: vec![Function::Posten],
            user_area: 1,
        };
        let response = test_get(&context.db_pool, app, &config).await;

        assert_eq!(StatusCode::OK, response.status());

        let body = read_body(response).await;
        assert_snapshot!(body);
    }

    #[db_test]
    async fn returns_unauthorized_when_user_is_not_admin(context: &DbTestContext) {
        let app = context.app().await;

        let config = RequestConfig {
            uri: "/area/new".to_string(),
            role: Role::AreaManager,
            function: vec![Function::Posten],
            user_area: 1,
        };
        let response = test_get(&context.db_pool, app, &config).await;

        assert_eq!(StatusCode::UNAUTHORIZED, response.status());
    }
}
